# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from stix.test import EntityTestCase
from stix.test.common import structured_text_tests

from stix import common


class ToolInformationTests(EntityTestCase, unittest.TestCase):
    klass = common.ToolInformation

    _full_dict = {
        'id': 'example:test-1',   # from python-cybox
        'name': 'Test Tool Name',  # from python-cybox
        'title': 'Test Title',
        'short_description': 'Test Short Description'
    }


class ToolInformationMultiDescTests(EntityTestCase, unittest.TestCase):
    klass = common.ToolInformation

    _full_dict = {
        'short_description': structured_text_tests.StructuredTextListTests._full_dict
    }

if __name__ == "__main__":
    unittest.main()
